C  ********************************************************************
C     Function to Calculate the Normalized Planck Function (B_nu/B)*(kT/h)
C  ********************************************************************
      real function Bnu(nu,T)
      
      implicit none

      real norm      
      parameter (norm=0.1539897)    ! 15/pi**4

      real nu,T,x
      real expm1

      x=nu/T
      
      if (x.eq.0.) then
          Bnu=0.
      else
          Bnu=norm*x*x*x/expm1(x)  !Bx = (B_nu/B)*(kT/h)
      endif
      
      return
      end



C  ********************************************************************
C     Function to Calculate the temperature derivative of the Normalized 
C     Planck Function (dB_nu/dT)/(dB/dT) * (kT/h)
C  ********************************************************************
      real function dBnu(nu,T)
      
      implicit none

      real norm      
      parameter (norm=0.1539897)    ! 15/pi**4

      real nu,T,x
      real expm1

      x=nu/T
      
      if (x.eq.0.) then
          dBnu=0.
      else
          dBnu=-norm*x*x*x*x/(4.*expm1(x)*expm1(-x))  !x*Bx/(1-exp(-x))/4
      endif
      
      return
      end


C***************************************************************************
C Inverse error function complent.  It uses the ration func approx in 
C Abramowitz and Stegun (AMS-55), eq. 26.2.23, page 933. (Note sqrt(2) diff 
C between arguments of Q and erfc)
C
C***************************************************************************
      real function ierfc(x)
      
      implicit none
      
      real x,t,c0,c1,c2,d1,d2,d3,sqhalf
      
      data sqhalf /0.70710678/
      data c0,c1,c2 /2.515517,0.802853,0.010328/
      data d1,d2,d3 /1.432788,0.189269,0.001308/
      
      t=sqrt(2.*log(2./x))
      
      ierfc=(t-(c0+t*(c1+t*c2))/(1.+t*(d1+t*(d2+t*d3))))*sqhalf
      
      return
      end


      real FUNCTION EXPM1(x)
C  ********************************************************************
C     THIS FUNCTION COMPUTES THE VALUE OF EXP(X)-1 USING THE
C     EXPANSION IN ABRAMOWITZ AND STEGUN (SEC. 4.2.45).  THIS
C     IS USED TO GET AROUND THE ACCURACY PROBLEMS INHERENT
C     WHEN SUBTRACTING EXP(X) - 1 WHEN X IS VERY
C     SMALL.
C  ********************************************************************

      real x,maxarg
      REAL*8 ARG,COEF,SUM
      INTEGER I
      DIMENSION COEF(7)

      DATA COEF/0.9999999995D0,0.4999999206D0,0.1666653019D0,
     1    0.0416573475D0,0.0083013598D0,0.0013298820D0,0.0001413161D0/
      DATA maxarg/0.6931472/

      ARG=x
      
      IF (abs(x) .GT. maxarg) THEN
           EXPM1 = DEXP(ARG)-1.0D0
      ELSE
           SUM = 0.D0
           DO 10 I=7,1,-1
                SUM = ARG*(COEF(I)+SUM)
   10      CONTINUE
           EXPM1 = SUM
      ENDIF

      RETURN
      END

      
      
      SUBROUTINE qsimp(func,a,b,s)
      INTEGER JMAX
      REAL a,b,func,s,EPS
      EXTERNAL func
      PARAMETER (EPS=1.e-5, JMAX=20)
CU    USES trapzd
      INTEGER j
      REAL os,ost,st
      ost=-1.e30
      os= -1.e30
      do 11 j=1,JMAX
        call trapzd(func,a,b,st,j)
        s=(4.*st-ost)/3.
        if (abs(s-os).lt.EPS*abs(os)) return
        os=s
        ost=st
11    continue
      pause 'too many steps in qsimp'
      END

      SUBROUTINE trapzd(func,a,b,s,n)
      INTEGER n
      REAL a,b,s,func
      EXTERNAL func
      INTEGER it,j
      REAL del,sum,tnm,x
      if (n.eq.1) then
        s=0.5*(b-a)*(func(a)+func(b))
      else
        it=2**(n-2)
        tnm=it
        del=(b-a)/tnm
        x=a+0.5*del
        sum=0.
        do 11 j=1,it
          sum=sum+func(x)
          x=x+del
11      continue
        s=0.5*(s+(b-a)*sum/tnm)
      endif
      return
      END
