c **********************************************************************
      subroutine atmosinit(atmosfile)
c ----------------------------------------------------------------------
c   Reads in a two-column array containing wavelengths and fluxes
c   Calculates partial integrals at every wavelength point
c   Normalizes the integrals on the bolometric flux
c   Uses real function tr2 for integration
c-----------------------------------------------------------------------
c                                [Anatoly Miroshnichenko, Mar. 1998]
c ======================================================================       
      
      implicit none
      
      include 'dust.com'
      
      integer nHnu,nHnumax
      parameter(nHnumax=25000)
      real atmosnu,Hnuint
      dimension atmosnu(nHnumax),Hnuint(nHnumax)
      common /atmos_interp/atmosnu,Hnuint,nHnu

      CHARACTER*80 atmosfile
      CHARACTER*80 header1,header2,header3
      INTEGER i,n
      REAL w(nHnumax),winv(nHnumax),f(nHnumax),chif(nHnumax),
     +     fbol,tr2
c-----------------------------------------------------------------------

      open(21,file=atmosfile,status='old')
      read(21,*) header1
      read(21,*) header2
      read(21,*) header3

      do i=1,nHnumax

          read(21,*,end=100) w(i),winv(i),f(i)

	  atmosnu(i)=1.24*winv(i)
          call opacset(atmosnu(i))
          chif(i)=kappa*f(i)

          if(i.eq.1) then
             Hnuint(i)=0.
          else
             Hnuint(i)=Hnuint(i-1)+0.5*(f(i)+f(i-1))*(winv(i-1)-winv(i))
          endif

      end do
      
      write(*,*) 'atmosinit: too many frequencies in model atmosphere'
      stop

 100  close(21)

      nHnu=i-1

      write(*,*) 'nHnu=',nHnu

      fbol=Hnuint(nHnu)
      kappaf=0.
      do i=1,nHnu
            Hnuint(i)=Hnuint(i)/fbol
            kappaf=kappaf+0.5*(chif(i)+chif(i-1))*(winv(i-1)-winv(i))
      end do
      kappaf=kappaf/fbol

      fbol=fbol*2.997925e14
      write(*,*) 'Teff,kappaf=',(4.*3.141593*fbol/5.67051e-5)**0.25,
     +           kappaf

      return
      end

      

      real function atmosfreq()

      implicit none

      include 'random.com'
      
      integer i0,i1,i
      real xi,f,P0,P1,P

      integer nHnu,nHnumax
      parameter(nHnumax=25000)
      real atmosnu,Hnuint
      dimension atmosnu(nHnumax),Hnuint(nHnumax)
      common /atmos_interp/atmosnu,Hnuint,nHnu

      real ran2

      
      xi=ran2(idum)

      i0=1
      i1=nHnu

      P0=  Hnuint(i0)
      if(xi.lt.P0) then
          atmosfreq=atmosnu(1)
	  return
      endif

      P1=  Hnuint(i1)
      if (P1 .gt. 1.) then
          write(*,*) 'atmosfreq: P1>1, P1 = ',P1
	  P1=1.
      endif
      if (xi.ge.P1) then
          atmosfreq=atmosnu(nHnu)
          return
      endif

      do while((i1-i0).gt.1)

          i=(i0+i1)/2
          P=Hnuint(i)

	  if (P .gt. 1.) then
              write(*,*)'atmosfreq: P>1; i,P = ',i,P
	      P=1.
	  endif

          if (P .lt. P0) then
	      write(*,*) 'atmosfreq: P<P0; P0,P,P1 = ',P0,P,P1
	      P0=P
	      i0=i
	  endif
          if (P .gt. P1) then
	      write(*,*) 'atmosfreq: P>P1; P0,P,P1 = ',P0,P,P1
	      P1=P
	      i1=1
	  endif
	  
          if (P.gt.xi) then
              i1=i
              P1=P
          else
              i0=i
              P0=P
          endif

      end do

      if (P1.le.P0) then
          write(*,*) 'atmosfreq: P1<=P0; P0, P1 = ',P0,P1
	  stop
      endif
      
c      f=(xi-P0)/(P1-P0)
      if (xi.lt.P0) then
          write(*,*) 'atmosfreq: xi<P0, xi,P0 = ',xi,P0
          f=0.
      elseif (xi.gt.P1) then
          write(*,*) 'atmosfreq: xi>P1, xi,P1 = ',xi,P1
          f=1.
      elseif (P0.le.0.) then
          write(*,*) 'atmosfreq: P0<=0, P0,i0,i1 = ',P0,i0,i1
	  f=0.5
      elseif (P1.le.0.) then
          write(*,*) 'atmosfreq: P1<=0, P1 = ',P1
	  stop
      elseif (xi.le.0.) then
          write(*,*) 'atmosfreq: xi<=0, xi = ',xi
	  stop
      else
          f=log(xi/P0)/log(P1/P0)
      endif
      if ((f.lt.-0.0001).or.(f.ge.1.0001)) then
          write(*,*) 'atmosfreq: f = ',f,' out of bounds'
      endif

      atmosfreq= atmosnu(i0) * (atmosnu(i1)/atmosnu(i0))**f

      print*,'xi,Hnuint,atmosfreq',xi,Hnuint(i0),atmosnu(i0),atmosfreq

      return
      end
